# coding: utf-8
"""
    PyLucid Admin App. settings
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    settings witch only used in the PyLucid Admin
"""

URL_FILENAME = "admin_urls.py"

VIEW_FILENAME = "admin_views"

PLUGIN_INSTALL_VIEW_NAME = "install"

# The PyLucid admin user group:
#PYLUCID_ADMIN_GROUP = "PyLucid Admin"
USER_GROUP = "PyLucid Admin"

# The design name for the PyLucid admin section:
DESIGN_NAME = "PyLucid Admin"
